import 'dart:async';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:nuntio_hera/hera/models/find_options.dart';
import 'package:pinput/pinput.dart';

import '../../../../hera.pb.dart';
import '../../../../hera.pbenum.dart';
import '../../../hera/hera.dart';
import '../../components/nuntio_indicator.dart';
import '../../models.dart';

class VerifyCodePage extends StatefulWidget {
  VerifyCodePage({
    Key? key,
    required this.nuntioFooter,
    required this.nuntioStyle,
    required this.nuntioText,
    required this.nuntioColor,
    required this.nuntioTextStyle,
    required this.logo,
    required this.config,
    required this.background,
    required this.password,
    required this.expiresAt,
    required this.loginType,
    required this.userId,
  }) : super(key: key);

  final String userId;
  final String password;
  final DateTime expiresAt;

  // style and text config
  final HeraStyle nuntioStyle;
  final HeraText nuntioText;
  final HeraColor nuntioColor;
  final HeraTextStyle nuntioTextStyle;
  final HeraFooter nuntioFooter;
  final LoginType loginType;

  // general
  final Widget logo;
  final Config config;

  // style
  final BoxDecoration background;

  @override
  State<VerifyCodePage> createState() => _VerifyCodePageState();
}

class _VerifyCodePageState extends State<VerifyCodePage> {
  final verifyCodeController = TextEditingController();
  DateTime _now = DateTime.now();
  bool _isLoading = false;
  bool _hasError = false;
  late Timer _timer;

  _VerifyCodePageState() {
    _timer = Timer.periodic(Duration(seconds: 1), (Timer t) {
      setState(() {
        _now = DateTime.now();
      });
      if (widget.expiresAt.difference(_now).inSeconds <= 0) {
        Navigator.of(context).popUntil((route) => route.isFirst);
      }
    });
  }

  @override
  void dispose() {
    _timer.cancel();
    super.dispose();
  }

  String formatTime(int totalSeconds) {
    final duration = Duration(seconds: totalSeconds);
    final minutes = duration.inMinutes;
    final seconds = totalSeconds % 60;

    final minutesString = '$minutes'.padLeft(2, '0');
    final secondsString = '$seconds'.padLeft(2, '0');
    return '$minutesString:$secondsString';
  }

  Future<void> verifyCode() async {
    if (widget.loginType == LoginType.EMAIL_PASSWORD) {
      return await Hera.verifyEmail(
          code: verifyCodeController.text,
          findOptions: FindOptions(id: widget.userId));
    } else if (widget.loginType == LoginType.PHONE_PASSWORD) {
      return await Hera.verifyPhone(
          code: verifyCodeController.text,
          findOptions: FindOptions(id: widget.userId));
    }
  }

  onSubmit() {
    if (verifyCodeController.text.length == 6) {
      setState(() {
        _isLoading = true;
      });
      verifyCode().catchError((err) {
        verifyCodeController.clear();
        print("could not verify email with err: " + err.toString());
        setState(() {
          _hasError = true;
          _isLoading = false;
        });
        return err;
      }).then((_) {
        verifyCodeController.clear();
        setState(() {
          _isLoading = false;
        });
        Hera.login(
                password: widget.password,
                findOptions: FindOptions(id: widget.userId))
            .then((loginSession) {
          if (loginSession.loginStatus == LoginStatus.PHONE_IS_NOT_VERIFIED) {
            Navigator.of(context)
                .pushReplacement(
              CupertinoPageRoute(
                builder: (context) => VerifyCodePage(
                  loginType: LoginType.PHONE_PASSWORD,
                  userId: widget.userId,
                  expiresAt: loginSession.expiresAt.toDateTime().toUtc(),
                  config: widget.config,
                  nuntioStyle: widget.nuntioStyle,
                  nuntioColor: widget.nuntioColor,
                  background: widget.background,
                  nuntioText: widget.nuntioText,
                  nuntioFooter: widget.nuntioFooter,
                  nuntioTextStyle: widget.nuntioTextStyle,
                  password: widget.password,
                  logo: widget.logo,
                ),
              ),
            )
                .then((_) {
              Navigator.of(context).popUntil((route) => route.isFirst);
            });
          } else {
            Navigator.of(context).popUntil((route) => route.isFirst);
          }
        }).catchError((err) {
          print(err);
        });
      });
    }
    if (_isLoading) {
      return;
    }
  }

  @override
  Widget build(BuildContext context) {
    var size = MediaQuery.of(context).size;
    return CupertinoPageScaffold(
      navigationBar: CupertinoNavigationBar(
        backgroundColor: Colors.transparent,
        border: Border(),
        leading: CupertinoNavigationBarBackButton(
          color: widget.nuntioColor.primaryColor,
          onPressed: () =>
              {Navigator.of(context).popUntil((route) => route.isFirst)},
        ),
      ),
      //resizeToAvoidBottomInset: false,
      child: SingleChildScrollView(
        child: Container(
          decoration: widget.background,
          constraints: BoxConstraints(minHeight: size.height),
          child: SafeArea(
            child: Column(
              crossAxisAlignment: CrossAxisAlignment.center,
              mainAxisAlignment: MainAxisAlignment.spaceBetween,
              children: [
                Center(
                  child: Container(
                    constraints: BoxConstraints(maxWidth: 450),
                    margin: const EdgeInsets.only(
                        left: 20, right: 20, bottom: 20, top: 0),
                    child: Column(
                      crossAxisAlignment: CrossAxisAlignment.center,
                      mainAxisAlignment: MainAxisAlignment.center,
                      children: [
                        widget.logo,
                        SizedBox(
                          height: 20,
                        ),
                        Text(
                          widget.loginType == LoginType.EMAIL_PASSWORD
                              ? widget.nuntioText.verifyEmailTitle
                              : widget.nuntioText.verifyPhoneTitle,
                            style: widget.nuntioTextStyle.titleStyle,
                            textAlign: TextAlign.center,
                          ),
                        SizedBox(
                          height: 10,
                        ),
                        Text(
                          widget.loginType == LoginType.EMAIL_PASSWORD
                              ? widget.nuntioText.verifyEmailSubtitle
                              : widget.nuntioText.verifyPhoneSubtitle,
                          style: widget.nuntioTextStyle.subtitleStyle,
                          textAlign: TextAlign.center,
                        ),
                        SizedBox(
                          height: 50,
                        ),
                        Text(
                          formatTime(
                              widget.expiresAt.difference(_now).inSeconds),
                          style: Theme.of(context)
                              .textTheme
                              .displaySmall
                              ?.copyWith(color: CupertinoColors.black),
                        ),
                        SizedBox(
                          height: 50,
                        ),
                        Card(
                          elevation: 0,
                          child: Pinput(
                            length: 6,
                            obscureText: false,
                            enabled: true,
                            hapticFeedbackType: HapticFeedbackType.mediumImpact,
                            // set this based on expiration
                            keyboardType: TextInputType.number,
                            animationDuration: Duration(milliseconds: 300),
                            controller: verifyCodeController,
                            onSubmitted: (_) {
                              onSubmit();
                            },
                            textInputAction: TextInputAction.done,
                            androidSmsAutofillMethod:
                                AndroidSmsAutofillMethod.smsRetrieverApi,
                          ),
                        ),
                        if (!_hasError)
                          SizedBox(
                            height: 50,
                          ),
                        if (_hasError)
                          SizedBox(
                            height: 25,
                          ),
                        if (_hasError)
                          Text(
                              "Something went wrong. Please try again and make sure your code is valid.",
                              style: Theme.of(context).textTheme.bodyText1),
                        if (_hasError)
                          SizedBox(
                            height: 25,
                          ),
                        SizedBox(
                          width: widget.nuntioStyle.buttonWidth,
                          height: widget.nuntioStyle.buttonHeight,
                          child: CupertinoButton(
                            child: _isLoading
                                ? NuntioIndicator(
                                    color: CupertinoColors.white,
                                  )
                                : Text(
                                    widget.nuntioText.verifyButtonText,
                                    style:
                                        TextStyle(color: CupertinoColors.white),
                                  ),
                            color: widget.nuntioColor.primaryColor,
                            onPressed: onSubmit,
                          ),
                        ),
                      ],
                    ),
                  ),
                ),
                SizedBox(
                  height: widget.nuntioFooter.height,
                  child: widget.nuntioFooter.footer,
                ),
              ],
            ),
          ),
        ),
      ),
    );
  }
}
